'use strict';

const mocha = require('mocha');

const describe = mocha.describe;
const it = mocha.it;
const beforeEach = mocha.beforeEach;

const should = require('should');
const {
  extractSnippets,
  decorateJSParserWithVueSupport,
  decorateExtractorWithHelpers,
  ensureSingleLine,
} = require('./index');
const fs = require('fs');
const glob = require('glob');

const errorMessage = (actual, expected) =>
  `Expected ${JSON.stringify(actual, null, 2)} to match ${JSON.stringify(expected, null, 2)}`;

describe('extractSnippets Tests', () => {
  const tests = glob.sync('fixtures/*.{vue,js}');

  tests.forEach(file => {
    it(`should parse ${file} correctly`, () => {
      const comparison = JSON.parse(fs.readFileSync(`${file}.snippets.json`, 'utf8'));
      return extractSnippets(file).then(snippets => {
        should(snippets).deepEqual(comparison, errorMessage(snippets, comparison));
      });
    });
  });
});

describe('decorateJSParserWithVueSupport', () => {
  const { GettextExtractor, JsExtractors } = require('gettext-extractor');
  let vueParser;
  let extractor;

  beforeEach(() => {
    extractor = new GettextExtractor();

    const jsParser = extractor.createJsParser([
      // Place all the possible expressions to extract here:
      JsExtractors.callExpression('__', {
        arguments: {
          text: 0,
        },
      }),
      JsExtractors.callExpression('n__', {
        arguments: {
          text: 0,
          textPlural: 1,
        },
      }),
      JsExtractors.callExpression('s__', {
        arguments: {
          text: 0,
        },
      }),
    ]);

    vueParser = decorateJSParserWithVueSupport(jsParser);
  });

  describe('parseString', () => {
    it('should parse single line strings correctly', () => {
      return vueParser.parseString('s__("text")', 'a.js').then(() => {
        should(extractor.getMessages()).deepEqual([
          {
            text: 'text',
            textPlural: null,
            context: null,
            references: ['a.js:1'],
            comments: [],
          },
        ]);
      });
    });

    it('should parse multiline strings correctly', () => {
      return vueParser.parseString('n__("text", "texts")', 'a.js').then(() => {
        should(extractor.getMessages()).deepEqual([
          {
            text: 'text',
            textPlural: 'texts',
            context: null,
            references: ['a.js:1'],
            comments: [],
          },
        ]);
      });
    });
  });

  describe('parseFile', () => {
    const tests = glob.sync('fixtures/*.{vue,js}');
    tests.forEach(file => {
      it(`should parse ${file} correctly`, () => {
        const comparison = JSON.parse(fs.readFileSync(`${file}.messages.json`, 'utf8'));
        return vueParser.parseFile(file).then(() => {
          should(extractor.getMessages()).deepEqual(
            comparison,
            errorMessage(extractor.getMessages(), comparison)
          );
        });
      });
    });
  });

  describe('parseFilesGlob', () => {
    it(`should parse all files matching the 'fixtures/*.vue' glob`, () => {
      const comparison = JSON.parse(fs.readFileSync('fixtures/all.vue.json', 'utf8'));
      return vueParser.parseFilesGlob('fixtures/*.vue').then(() => {
        should(extractor.getMessages()).deepEqual(
          comparison,
          errorMessage(extractor.getMessages(), comparison)
        );
      });
    });
  });
});

describe('decorateExtractorWithHelpers', () => {
  const { GettextExtractor, JsExtractors } = require('gettext-extractor');
  let extractor;

  beforeEach(() => {
    extractor = new GettextExtractor();

    extractor = decorateExtractorWithHelpers(extractor);
  });

  describe('ensureSingleLineMessages', () => {
    it('should remove unnecessary new lines from extractor.addMessage', () => {
      extractor.ensureSingleLineMessages();

      extractor.addMessage({
        text: '\nTest\n',
        textPlural: '\nPlural\nspans\n  multiple lines.\n',
      });

      should(extractor.getMessages()).deepEqual([
        {
          text: 'Test',
          textPlural: 'Plural spans multiple lines.',
          context: null,
          references: [],
          comments: [],
        },
      ]);
    });

    it('should remove unnecessary new lines from parser.parseString', () => {
      extractor.ensureSingleLineMessages();

      const jsParser = extractor.createJsParser([
        JsExtractors.callExpression('__', {
          arguments: {
            text: 0,
            textPlural: 1,
          },
        }),
      ]);

      jsParser.parseString('__(`\n Test\r\n message\n`, ` Plural`)', 'a.js');

      should(extractor.getMessages()).deepEqual([
        {
          text: 'Test message',
          textPlural: ' Plural',
          context: null,
          references: ['a.js:1'],
          comments: [],
        },
      ]);
    });

    it('should throw error if string has been parsed before activating', () => {
      extractor.addMessage({ text: 'Test' });

      should.throws(() => {
        extractor.ensureSingleLineMessages();
      }, /^Error: Make sure you run ensureSingleLineMessages/);
    });
  });
});

describe('ensureSingleLine', () => {
  it('should remove newlines at the start of the string', () => {
    const result = 'Test';
    should(ensureSingleLine(`\n${result}`)).equal(result);
    should(ensureSingleLine(`\t\n\t${result}`)).equal(result);
    should(ensureSingleLine(`\r\n${result}`)).equal(result);
    should(ensureSingleLine(`\r\n ${result}`)).equal(result);
    should(ensureSingleLine(`\r ${result}`)).equal(result);
    should(ensureSingleLine(` \n ${result}`)).equal(result);
  });

  it('should remove newlines at the end of the string', () => {
    const result = 'Test';
    should(ensureSingleLine(`${result}\n`)).equal(result);
    should(ensureSingleLine(`${result}\t\n\t`)).equal(result);
    should(ensureSingleLine(`${result}\r\n`)).equal(result);
    should(ensureSingleLine(`${result}\r`)).equal(result);
    should(ensureSingleLine(`${result} \r`)).equal(result);
    should(ensureSingleLine(`${result} \r\n `)).equal(result);
  });

  it('should replace newlines in the middle of the string with a single space', () => {
    const result = 'Test';
    should(ensureSingleLine(`${result}\n${result}`)).equal(`${result} ${result}`);
    should(ensureSingleLine(`${result}\t\n\t${result}`)).equal(`${result} ${result}`);
    should(ensureSingleLine(`${result}\r\n${result}`)).equal(`${result} ${result}`);
    should(ensureSingleLine(`${result}\r${result}`)).equal(`${result} ${result}`);
    should(ensureSingleLine(`${result} \r${result}`)).equal(`${result} ${result}`);
    should(ensureSingleLine(`${result} \r\n ${result}`)).equal(`${result} ${result}`);
  });
});
